import { Directive, ElementRef, Host, HostBinding, HostListener, Input, NgModule } from '@angular/core';
import { Subject as Subject$1 } from 'rxjs/Subject';
import 'rxjs/add/operator/takeUntil';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import 'rxjs/add/operator/filter';
var ContentsDirective = (function () {
    function ContentsDirective() {
        this._activeSection$ = new BehaviorSubject$1(null);
    }
    /**
     * @return {?}
     */
    ContentsDirective.prototype.activeSection = function () {
        return this._activeSection$
            .asObservable()
            .filter(function (section) { return !!section; }); // Prevent returning null values.
    };
    return ContentsDirective;
}());
ContentsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contents]',
                exportAs: 'contents',
            },] },
];
/**
 * @nocollapse
 */
ContentsDirective.ctorParameters = function () { return []; };
var ContentsLinkDirective = (function () {
    /**
     * @param {?} contents
     */
    function ContentsLinkDirective(contents) {
        this.contents = contents;
        this.ngUnsubscribe = new Subject$1();
        this.active = false;
    }
    /**
     * @return {?}
     */
    ContentsLinkDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.contents._activeSection$
            .takeUntil(this.ngUnsubscribe)
            .subscribe(function (sectionName) {
            _this.active = "#" + sectionName === _this.href;
        });
    };
    /**
     * @return {?}
     */
    ContentsLinkDirective.prototype.ngOnDestroy = function () {
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    };
    return ContentsLinkDirective;
}());
ContentsLinkDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsLink]',
                exportAs: 'contentsLink',
            },] },
];
/**
 * @nocollapse
 */
ContentsLinkDirective.ctorParameters = function () { return [
    { type: ContentsDirective, decorators: [{ type: Host },] },
]; };
ContentsLinkDirective.propDecorators = {
    'href': [{ type: Input },],
    'active': [{ type: HostBinding, args: ['class.active',] },],
};
/**
 * @param {?} element
 * @return {?}
 */
function getAbsoluteHeight(element) {
    var /** @type {?} */ styles = window.getComputedStyle(element);
    var /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
        parseFloat(styles.marginBottom || '0');
    return Math.ceil(element.offsetHeight + margin);
}
var ContentsSectionDirective = (function () {
    /**
     * @param {?} contents
     * @param {?} elementRef
     */
    function ContentsSectionDirective(contents, elementRef) {
        this.contents = contents;
        this.elementRef = elementRef;
    }
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.ngOnInit = function () {
        this.detectActiveChanges();
    };
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.detectActiveChanges = function () {
        if (this.isInRange() || !this.contents._activeSection$.value) {
            this.contents._activeSection$.next(this.contentsSection);
        }
    };
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.isInRange = function () {
        var /** @type {?} */ pageOffset = window.pageYOffset;
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ offset = element.offsetTop;
        var /** @type {?} */ height = getAbsoluteHeight(element);
        return pageOffset >= offset && pageOffset <= offset + height;
    };
    return ContentsSectionDirective;
}());
ContentsSectionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsSection]',
                exportAs: 'contentsSection',
            },] },
];
/**
 * @nocollapse
 */
ContentsSectionDirective.ctorParameters = function () { return [
    { type: ContentsDirective, decorators: [{ type: Host },] },
    { type: ElementRef, },
]; };
ContentsSectionDirective.propDecorators = {
    'contentsSection': [{ type: HostBinding, args: ['id',] }, { type: Input },],
    'detectActiveChanges': [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
};
var ContentsTableDirective = (function () {
    /**
     * @param {?} elementRef
     */
    function ContentsTableDirective(elementRef) {
        this.elementRef = elementRef;
        this.sticky = false;
        this.marginTop = '0px';
    }
    /**
     * @return {?}
     */
    ContentsTableDirective.prototype.ngOnInit = function () {
        this.updateStickiness();
    };
    /**
     * Check whether the Table of Contents should be a sticky, to keep itself visible while the user
     * scrolls.
     * @return {?}
     */
    ContentsTableDirective.prototype.updateStickiness = function () {
        var /** @type {?} */ pageOffset = window.pageYOffset;
        var /** @type {?} */ parentElement = this.elementRef.nativeElement.parentNode;
        var /** @type {?} */ parentOffset = parentElement.offsetTop;
        var /** @type {?} */ parentHeight = getAbsoluteHeight(parentElement);
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ elementInnerHeight = element.offsetHeight;
        if (pageOffset + elementInnerHeight > parentOffset + parentHeight) {
            // Use a fixed margin-top instead when scrolling past the parent container.
            this.sticky = false;
            this.marginTop = parentHeight - elementInnerHeight + "px";
        }
        else {
            // Use CSS sticky when scrolling into the parent container.
            this.marginTop = '0px';
            this.sticky = pageOffset > parentOffset;
        }
    };
    /**
     * @param {?} target
     * @return {?}
     */
    ContentsTableDirective.prototype.getHeight = function (target) {
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ styles = window.getComputedStyle(element);
        var /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
            parseFloat(styles.marginBottom || '0');
        return Math.ceil(element.offsetHeight + margin);
    };
    return ContentsTableDirective;
}());
ContentsTableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsTable]',
                exportAs: 'contentsTable',
            },] },
];
/**
 * @nocollapse
 */
ContentsTableDirective.ctorParameters = function () { return [
    { type: ElementRef, },
]; };
ContentsTableDirective.propDecorators = {
    'sticky': [{ type: HostBinding, args: ['class.sticky',] },],
    'marginTop': [{ type: HostBinding, args: ['style.margin-top',] },],
    'updateStickiness': [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
};
var ContentsModule = (function () {
    function ContentsModule() {
    }
    return ContentsModule;
}());
ContentsModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                declarations: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ContentsModule.ctorParameters = function () { return []; };
/**
 * Generated bundle index. Do not edit.
 */
export { ContentsLinkDirective, ContentsSectionDirective, ContentsTableDirective, ContentsDirective, ContentsModule };
//# sourceMappingURL=angular-contents.es5.js.map
